export const PAGEBREAK_CELL_TAG = 'jp-pb-pagebreakCell';

export const PAGEBREAK_HEADER_TAG = 'jp-pb-header';

export const DROP_TARGET_CLASS_BOTTOM = 'jp-mod-dropTarget-bottom';

export const BAD_FORMATTING_TAG = 'jp-pb-badFormatting';
